@@ -1,6 +1,6 @@
 /*
     SDL_net:  An example cross-platform network library for use with SDL
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
     slouken@libsdl.org
 */
 
-/* $Id: SDLnetTCP.c,v 1.11 2002/04/13 14:50:37 slouken Exp $ */
+/* $Id: SDLnetTCP.c 2425 2006-05-14 05:59:44Z slouken $ */
 
 #include "SDLnetsys.h"
 #include "SDL_net.h"
@@ -588,9 +588,9 @@ IPaddress *SDLNet_TCP_GetPeerAddress(TCPsocket sock)
    is less than the amount of data sent, then either the remote connection was
    closed, or an unknown socket error occurred.
 */
-int SDLNet_TCP_Send(TCPsocket sock, void *datap, int len)
+int SDLNet_TCP_Send(TCPsocket sock, const void *datap, int len)
 {
-	Uint8 *data = (Uint8 *)datap;	/* For pointer arithmetic */
+	const Uint8 *data = (const Uint8 *)datap;	/* For pointer arithmetic */
 	int sent, left;
 
 	/* Server sockets are for accepting connections only */
@@ -604,7 +604,7 @@ int SDLNet_TCP_Send(TCPsocket sock, void *datap, int len)
 	sent = 0;
 	errno = 0;
 	do {
-		len = OTSnd(sock->channel, data, left, 0);
+		len = OTSnd(sock->channel, (void *)data, left, 0);
 		if (len == kOTFlowErr)
 			len = 0;
 		if ( len > 0 ) {
@@ -740,10 +740,17 @@ TCPsocket SDLNet_TCP_Open(IPaddress *ip)
 		sock_addr.sin_addr.s_addr = INADDR_ANY;
 		sock_addr.sin_port = ip->port;
 
+/*
+ * Windows gets bad mojo with SO_REUSEADDR:
+ * http://www.devolution.com/pipermail/sdl/2005-September/070491.html
+ *   --ryan.
+ */
+#ifndef WIN32
 		/* allow local address reuse */
 		{ int yes = 1;
 			setsockopt(sock->channel, SOL_SOCKET, SO_REUSEADDR, (char*)&yes, sizeof(yes));
 		}
+#endif
 
 		/* Bind the socket for listening */
 		if ( bind(sock->channel, (struct sockaddr *)&sock_addr,
@@ -755,21 +762,31 @@ TCPsocket SDLNet_TCP_Open(IPaddress *ip)
 			SDLNet_SetError("Couldn't listen to local port");
 			goto error_return;
 		}
-#ifdef O_NONBLOCK
+
 		/* Set the socket to non-blocking mode for accept() */
-		fcntl(sock->channel, F_SETFL, O_NONBLOCK);
-#else
-#ifdef WIN32
+#if defined(__BEOS__) && defined(SO_NONBLOCK)
+		/* On BeOS r5 there is O_NONBLOCK but it's for files only */
 		{
-		  /* passing a non-zero value, socket mode set non-blocking */
-		  unsigned long mode = 1;
-		  ioctlsocket (sock->channel, FIONBIO, &mode);
+			long b = 1;
+			setsockopt(sock->channel, SOL_SOCKET, SO_NONBLOCK, &b, sizeof(b));
+		}
+#elif defined(O_NONBLOCK)
+		{
+			fcntl(sock->channel, F_SETFL, O_NONBLOCK);
+		}
+#elif defined(WIN32)
+		{
+			unsigned long mode = 1;
+			ioctlsocket (sock->channel, FIONBIO, &mode);
+		}
+#elif defined(__OS2__)
+		{
+			int dontblock = 1;
+			ioctl(sock->channel, FIONBIO, &dontblock);
 		}
 #else
 #warning How do we set non-blocking mode on other operating systems?
-#endif /* WIN32 */
-#endif /* O_NONBLOCK */
-
+#endif
 		sock->sflag = 1;
 	}
 	sock->ready = 0;
@@ -865,9 +882,9 @@ IPaddress *SDLNet_TCP_GetPeerAddress(TCPsocket sock)
    is less than the amount of data sent, then either the remote connection was
    closed, or an unknown socket error occurred.
 */
-int SDLNet_TCP_Send(TCPsocket sock, void *datap, int len)
+int SDLNet_TCP_Send(TCPsocket sock, const void *datap, int len)
 {
-	Uint8 *data = (Uint8 *)datap;	/* For pointer arithmetic */
+	const Uint8 *data = (const Uint8 *)datap;	/* For pointer arithmetic */
 	int sent, left;
 
 	/* Server sockets are for accepting connections only */
